package ngjsvm

import (
	"fmt"
	"github.com/dop251/goja"
)

type JSRtNest struct {
	isReg             bool
	env               *JSEnv
	subRtMap          map[string]JsRtInterface
	alreadyRegistedRt []JsRtInterface
}

func NewJSRtNest(rts map[string]JsRtInterface) *JSRtNest {
	return &JSRtNest{
		isReg:             false,
		env:               nil,
		subRtMap:          rts,
		alreadyRegistedRt: make([]JsRtInterface, 0, len(rts)),
	}
}

func (j *JSRtNest) Dispose() {
	for _, v := range j.alreadyRegistedRt {
		if v != nil {
			v.Dispose()
		}
	}
}

func (j *JSRtNest) disposalIfFail() {
	if !j.isReg {
		j.Dispose()
	}
}

func (j *JSRtNest) RegisterRt(name string, env *JSEnv) (goja.Value, error) {
	j.env = env
	obj := j.env.EmptyObject()
	defer j.disposalIfFail()
	for k, v := range j.subRtMap {
		if v == nil {
			return nil, fmt.Errorf("nest rt register erorr: sub rt '%s' is nil", k)
		}
		if !v.IsRegistered() {
			rti, err := v.RegisterRt(k, j.env)
			if err != nil {
				return nil, fmt.Errorf("nest rt register erorr: sub rt '%s' register failed: %v", k, err)
			}
			err = obj.Set(k, rti)
			if err != nil {
				return nil, fmt.Errorf("nest rt register erorr: sub rt '%s' cast to js failed: %v", k, err)
			}
			j.alreadyRegistedRt = append(j.alreadyRegistedRt, v)
		} else {
			return nil, fmt.Errorf("nest rt register erorr: sub rt '%s' has already registered", k)
		}
	}
	j.isReg = true
	return obj, nil
}

func (j JSRtNest) IsRegistered() bool {
	return j.isReg
}
